@@ -49,7 +49,7 @@ test('can pass projects or global config', () => {
         getHasteName(filename) {
           return filename
             .substr(filename.lastIndexOf(path.sep) + 1)
-            .replace(/\.js$/, '');
+            .replace(/\\.js$/, '');
         },
       };
     `,
@@ -346,19 +346,19 @@ test('resolves projects and their <rootDir> properly', () => {
       },
     }),
     'project1.conf.json': JSON.stringify({
-      name: 'project1',
+      id: 'project1',
       rootDir: './project1',
       // root dir should be this project's directory
       setupFiles: ['<rootDir>/project1_setup.js'],
       testEnvironment: 'node',
     }),
     'project1/__tests__/test.test.js':
-      "test('project1', () => expect(global.project1).toBe(true))",
+      "test('project1', () => expect(globalThis.project1).toBe(true))",
     'project1/project1_setup.js': 'global.project1 = true;',
     'project2/__tests__/test.test.js':
-      "test('project2', () => expect(global.project2).toBe(true))",
+      "test('project2', () => expect(globalThis.project2).toBe(true))",
     'project2/project2.conf.json': JSON.stringify({
-      name: 'project2',
+      id: 'project2',
       rootDir: '../', // root dir is set to the top level
       setupFiles: ['<rootDir>/project2/project2_setup.js'], // rootDir shold be of the
       testEnvironment: 'node',
@@ -446,11 +446,11 @@ test('Does transform files with the corresponding project transformer', () => {
     'project1/jest.config.js': `
       module.exports = {
         rootDir: './',
-        transform: {'file\.js': './transformer.js'},
+        transform: {'file\\.js': './transformer.js'},
       };`,
     'project1/transformer.js': `
       module.exports = {
-        process: () => 'module.exports = "PROJECT1";',
+        process: () => ({code: 'module.exports = "PROJECT1";'}),
         getCacheKey: () => 'PROJECT1_CACHE_KEY',
       }
     `,
@@ -461,11 +461,11 @@ test('Does transform files with the corresponding project transformer', () => {
     'project2/jest.config.js': `
       module.exports = {
         rootDir: './',
-        transform: {'file\.js': './transformer.js'},
+        transform: {'file\\.js': './transformer.js'},
       };`,
     'project2/transformer.js': `
       module.exports = {
-        process: () => 'module.exports = "PROJECT2";',
+        process: () => ({code: 'module.exports = "PROJECT2";'}),
         getCacheKey: () => 'PROJECT2_CACHE_KEY',
       }
     `,
@@ -514,13 +514,13 @@ describe("doesn't bleed module file extensions resolution with multiple workers"
 
     expect(configs).toHaveLength(2);
 
-    const [{name: name1}, {name: name2}] = configs;
+    const [{id: id1}, {id: id2}] = configs;
 
-    expect(name1).toEqual(expect.any(String));
-    expect(name2).toEqual(expect.any(String));
-    expect(name1).toHaveLength(32);
-    expect(name2).toHaveLength(32);
-    expect(name1).not.toEqual(name2);
+    expect(id1).toEqual(expect.any(String));
+    expect(id2).toEqual(expect.any(String));
+    expect(id1).toHaveLength(32);
+    expect(id2).toHaveLength(32);
+    expect(id1).not.toEqual(id2);
 
     const {stderr} = runJest(DIR, [
       '--no-watchman',
@@ -557,13 +557,13 @@ describe("doesn't bleed module file extensions resolution with multiple workers"
 
     expect(configs).toHaveLength(2);
 
-    const [{name: name1}, {name: name2}] = configs;
+    const [{id: id1}, {id: id2}] = configs;
 
-    expect(name1).toEqual(expect.any(String));
-    expect(name2).toEqual(expect.any(String));
-    expect(name1).toHaveLength(32);
-    expect(name2).toHaveLength(32);
-    expect(name1).not.toEqual(name2);
+    expect(id1).toEqual(expect.any(String));
+    expect(id2).toEqual(expect.any(String));
+    expect(id1).toHaveLength(32);
+    expect(id2).toHaveLength(32);
+    expect(id1).not.toEqual(id2);
 
     const {stderr} = runJest(DIR, ['--no-watchman', '-w=2']);
 
@@ -572,3 +572,41 @@ describe("doesn't bleed module file extensions resolution with multiple workers"
     expect(stderr).toMatch('PASS project2/__tests__/project2.test.js');
   });
 });
+
+describe('Babel config in individual project works in multi-project', () => {
+  it('Prj-1 works individually', () => {
+    const result = runJest('multi-project-babel/prj-1');
+    expect(result.stderr).toMatch('PASS ./index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+  it('Prj-2 works individually', () => {
+    const result = runJest('multi-project-babel/prj-2');
+    expect(result.stderr).toMatch('PASS ./index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+  it('Prj-3 works individually', () => {
+    const result = runJest('multi-project-babel/prj-3');
+    expect(result.stderr).toMatch('PASS src/index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+  it('Prj-4 works individually', () => {
+    const result = runJest('multi-project-babel/prj-4');
+    expect(result.stderr).toMatch('PASS src/index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+  it('Prj-5 works individually', () => {
+    const result = runJest('multi-project-babel/prj-5');
+    expect(result.stderr).toMatch('PASS src/index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+  it('All project work when running from multiproject', () => {
+    const result = runJest('multi-project-babel');
+    expect(result.stderr).toMatch('PASS prj-1/index.test.js');
+    expect(result.stderr).toMatch('PASS prj-2/index.test.js');
+    expect(result.stderr).toMatch('PASS prj-3/src/index.test.js');
+    expect(result.stderr).toMatch('PASS prj-4/src/index.test.js');
+    expect(result.stderr).toMatch('PASS prj-5/src/index.test.js');
+    expect(result.stderr).toMatch('PASS prj-3/src/index.test.js');
+    expect(result.exitCode).toBe(0);
+  });
+});
