@@ -7,7 +7,7 @@
 
 'use strict';
 
-const JSDOMEnvironment = require('jest-environment-jsdom');
+const JSDOMEnvironment = require('jest-environment-jsdom').default;
 
 const sleep = ms => new Promise(resolve => setTimeout(resolve, ms));
 
@@ -16,9 +16,9 @@ class TestEnvironment extends JSDOMEnvironment {
     await this.assertRunnerWaitsForHandleTestEvent(event);
 
     if (event.hook) {
-      console.log(event.name + ': ' + event.hook.type);
+      console.log(`${event.name}: ${event.hook.type}`);
     } else if (event.test) {
-      console.log(event.name + ': ' + event.test.name);
+      console.log(`${event.name}: ${event.test.name}`);
     } else {
       console.log(event.name);
     }
