@@ -6,7 +6,7 @@ const fs = require('fs');
 const globalConfigPath = path.join(__dirname, 'globalConfig.json');
 
 module.exports = async function () {
-  await global.__MONGOD__.stop();
+  await globalThis.__MONGOD__.stop();
 
   fs.unlinkSync(globalConfigPath);
 };
