@@ -5,6 +5,7 @@
  * LICENSE file in the root directory of this source tree.
  */
 
+import {equals, iterableEquality} from '@jest/expect-utils';
 import {getType, isPrimitive} from 'jest-get-type';
 import {
   DIM_COLOR,
@@ -21,13 +22,11 @@ import {
   printWithType,
   stringify,
 } from 'jest-matcher-utils';
-import {equals} from './jasmineUtils';
 import type {
   MatcherState,
   MatchersObject,
   SyncExpectationResult,
 } from './types';
-import {iterableEquality} from './utils';
 
 // The optional property of matcher context is true if undefined.
 const isExpand = (expand?: boolean): boolean => expand !== false;
@@ -65,7 +64,8 @@ const isEqualValue = (expected: unknown, received: unknown): boolean =>
 const isEqualCall = (
   expected: Array<unknown>,
   received: Array<unknown>,
-): boolean => isEqualValue(expected, received);
+): boolean =>
+  received.length === expected.length && isEqualValue(expected, received);
 
 const isEqualReturn = (expected: unknown, result: any): boolean =>
   result.type === 'return' && isEqualValue(expected, result.value);
@@ -80,10 +80,11 @@ const printNumberOfReturns = (
   countReturns: number,
   countCalls: number,
 ): string =>
-  `\nNumber of returns: ${printReceived(countReturns)}` +
-  (countCalls !== countReturns
-    ? `\nNumber of calls:   ${printReceived(countCalls)}`
-    : '');
+  `\nNumber of returns: ${printReceived(countReturns)}${
+    countCalls !== countReturns
+      ? `\nNumber of calls:   ${printReceived(countCalls)}`
+      : ''
+  }`;
 
 type PrintLabel = (string: string, isExpectedCall: boolean) => string;
 
@@ -96,7 +97,7 @@ const getRightAlignedPrinter = (label: string): PrintLabel => {
 
   return (string: string, isExpectedCall: boolean) =>
     (isExpectedCall
-      ? '->' + ' '.repeat(Math.max(0, index - 2 - string.length))
+      ? `->${' '.repeat(Math.max(0, index - 2 - string.length))}`
       : ' '.repeat(Math.max(index - string.length))) +
     string +
     suffix;
@@ -116,22 +117,20 @@ const printReceivedCallsNegative = (
 
   const label = 'Received:     ';
   if (isOnlyCall) {
-    return label + printReceivedArgs(indexedCalls[0], expected) + '\n';
+    return `${label + printReceivedArgs(indexedCalls[0], expected)}\n`;
   }
 
   const printAligned = getRightAlignedPrinter(label);
 
-  return (
-    'Received\n' +
-    indexedCalls.reduce(
-      (printed: string, [i, args]: IndexedCall) =>
+  return `Received\n${indexedCalls.reduce(
+    (printed: string, [i, args]: IndexedCall) =>
+      `${
         printed +
         printAligned(String(i + 1), i === iExpectedCall) +
-        printReceivedArgs(args, expected) +
-        '\n',
-      '',
-    )
-  );
+        printReceivedArgs(args, expected)
+      }\n`,
+    '',
+  )}`;
 };
 
 const printExpectedReceivedCallsPositive = (
@@ -174,42 +173,42 @@ const printExpectedReceivedCallsPositive = (
               difference.includes('+ Received')
             ) {
               // Omit annotation in case multiple args have diff.
-              lines.push(difference.split('\n').slice(3).join('\n') + ',');
+              lines.push(`${difference.split('\n').slice(3).join('\n')},`);
               continue;
             }
           }
         }
 
         if (i < expected.length) {
-          lines.push(EXPECTED_COLOR('- ' + stringify(expected[i])) + ',');
+          lines.push(`${EXPECTED_COLOR(`- ${stringify(expected[i])}`)},`);
         }
         if (i < received.length) {
-          lines.push(RECEIVED_COLOR('+ ' + stringify(received[i])) + ',');
+          lines.push(`${RECEIVED_COLOR(`+ ${stringify(received[i])}`)},`);
         }
       }
 
-      return lines.join('\n') + '\n';
+      return `${lines.join('\n')}\n`;
     }
 
-    return expectedLine + label + printReceivedArgs(received, expected) + '\n';
+    return `${expectedLine + label + printReceivedArgs(received, expected)}\n`;
   }
 
   const printAligned = getRightAlignedPrinter(label);
 
   return (
+    // eslint-disable-next-line prefer-template
     expectedLine +
     'Received\n' +
     indexedCalls.reduce((printed: string, [i, received]: IndexedCall) => {
       const aligned = printAligned(String(i + 1), i === iExpectedCall);
-      return (
+      return `${
         printed +
         ((i === iExpectedCall || iExpectedCall === undefined) &&
         isLineDiffableCall(expected, received)
           ? aligned.replace(': ', '\n') +
             printDiffCall(expected, received, expand)
-          : aligned + printReceivedArgs(received, expected)) +
-        '\n'
-      );
+          : aligned + printReceivedArgs(received, expected))
+      }\n`;
     }, '')
   );
 };
@@ -225,7 +224,7 @@ const printDiffCall = (
     .map((arg, i) => {
       if (i < expected.length) {
         if (isEqualValue(expected[i], arg)) {
-          return indentation + '  ' + printCommon(arg) + ',';
+          return `${indentation}  ${printCommon(arg)},`;
         }
 
         if (isLineDiffableArg(expected[i], arg)) {
@@ -238,25 +237,22 @@ const printDiffCall = (
           ) {
             // Display diff with indentation.
             // Omit annotation in case multiple args have diff.
-            return (
-              difference
-                .split('\n')
-                .slice(3)
-                .map(line => indentation + line)
-                .join('\n') + ','
-            );
+            return `${difference
+              .split('\n')
+              .slice(3)
+              .map(line => indentation + line)
+              .join('\n')},`;
           }
         }
       }
 
       // Display + only if received arg has no corresponding expected arg.
-      return (
+      return `${
         indentation +
         (i < expected.length
-          ? '  ' + printReceived(arg)
-          : RECEIVED_COLOR('+ ' + stringify(arg))) +
-        ','
-      );
+          ? `  ${printReceived(arg)}`
+          : RECEIVED_COLOR(`+ ${stringify(arg)}`))
+      },`;
     })
     .join('\n');
 
@@ -336,20 +332,22 @@ const printReceivedResults = (
   }
 
   if (isOnlyCall && (iExpectedCall === 0 || iExpectedCall === undefined)) {
-    return label + printResult(indexedResults[0][1], expected) + '\n';
+    return `${label + printResult(indexedResults[0][1], expected)}\n`;
   }
 
   const printAligned = getRightAlignedPrinter(label);
 
   return (
+    // eslint-disable-next-line prefer-template
     label.replace(':', '').trim() +
     '\n' +
     indexedResults.reduce(
       (printed: string, [i, result]: IndexedResult) =>
-        printed +
-        printAligned(String(i + 1), i === iExpectedCall) +
-        printResult(result, expected) +
-        '\n',
+        `${
+          printed +
+          printAligned(String(i + 1), i === iExpectedCall) +
+          printResult(result, expected)
+        }\n`,
       '',
     )
   );
@@ -380,6 +378,7 @@ const createToBeCalledMatcher = (matcherName: string) =>
     const pass = count > 0;
     const message = pass
       ? () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of calls: ${printExpected(0)}\n` +
@@ -394,6 +393,7 @@ const createToBeCalledMatcher = (matcherName: string) =>
             }, [])
             .join('\n')
       : () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of calls: >= ${printExpected(1)}\n` +
@@ -428,6 +428,7 @@ const createToReturnMatcher = (matcherName: string) =>
 
     const message = pass
       ? () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of returns: ${printExpected(0)}\n` +
@@ -447,6 +448,7 @@ const createToReturnMatcher = (matcherName: string) =>
               )}`
             : '')
       : () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of returns: >= ${printExpected(1)}\n` +
@@ -484,10 +486,12 @@ const createToBeCalledTimesMatcher = (matcherName: string) =>
 
     const message = pass
       ? () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of calls: not ${printExpected(expected)}`
       : () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of calls: ${printExpected(expected)}\n` +
@@ -522,6 +526,7 @@ const createToReturnTimesMatcher = (matcherName: string) =>
 
     const message = pass
       ? () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of returns: not ${printExpected(expected)}` +
@@ -531,6 +536,7 @@ const createToReturnTimesMatcher = (matcherName: string) =>
               )}`
             : '')
       : () =>
+          // eslint-disable-next-line prefer-template
           matcherHint(matcherName, receivedName, expectedArgument, options) +
           '\n\n' +
           `Expected number of returns: ${printExpected(expected)}\n` +
@@ -579,6 +585,7 @@ const createToBeCalledWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: not ${printExpectedArgs(expected)}\n` +
@@ -602,6 +609,7 @@ const createToBeCalledWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             printExpectedReceivedCallsPositive(
@@ -648,6 +656,7 @@ const createToReturnWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: not ${printExpected(expected)}\n` +
@@ -674,6 +683,7 @@ const createToReturnWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: ${printExpected(expected)}\n` +
@@ -723,6 +733,7 @@ const createLastCalledWithMatcher = (matcherName: string) =>
           indexedCalls.push([iLast, calls[iLast]]);
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: not ${printExpectedArgs(expected)}\n` +
@@ -757,6 +768,7 @@ const createLastCalledWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             printExpectedReceivedCallsPositive(
@@ -803,6 +815,7 @@ const createLastReturnedMatcher = (matcherName: string) =>
           indexedResults.push([iLast, results[iLast]]);
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: not ${printExpected(expected)}\n` +
@@ -840,6 +853,7 @@ const createLastReturnedMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `Expected: ${printExpected(expected)}\n` +
@@ -908,6 +922,7 @@ const createNthCalledWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `n: ${nth}\n` +
@@ -970,6 +985,7 @@ const createNthCalledWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `n: ${nth}\n` +
@@ -1034,6 +1050,7 @@ const createNthReturnedWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `n: ${nth}\n` +
@@ -1099,6 +1116,7 @@ const createNthReturnedWithMatcher = (matcherName: string) =>
           }
 
           return (
+            // eslint-disable-next-line prefer-template
             matcherHint(matcherName, receivedName, expectedArgument, options) +
             '\n\n' +
             `n: ${nth}\n` +
