@@ -66,44 +66,45 @@ afterEach(() => {
   (console.warn as unknown as jest.SpyInstance).mockRestore();
 });
 
-it('picks a name based on the rootDir', async () => {
+it('picks an id based on the rootDir', async () => {
   const rootDir = '/root/path/foo';
-  const expected = createHash('md5')
+  const expected = createHash('sha256')
     .update('/root/path/foo')
     .update(String(Infinity))
-    .digest('hex');
+    .digest('hex')
+    .substring(0, 32);
   const {options} = await normalize(
     {
       rootDir,
     },
     {} as Config.Argv,
   );
-  expect(options.name).toBe(expected);
+  expect(options.id).toBe(expected);
 });
 
-it('keeps custom project name based on the projects rootDir', async () => {
-  const name = 'test';
+it('keeps custom project id based on the projects rootDir', async () => {
+  const id = 'test';
   const {options} = await normalize(
     {
-      projects: [{name, rootDir: '/path/to/foo'}],
+      projects: [{id, rootDir: '/path/to/foo'}],
       rootDir: '/root/path/baz',
     },
     {} as Config.Argv,
   );
 
-  expect(options.projects[0].name).toBe(name);
+  expect(options.projects[0].id).toBe(id);
 });
 
-it('keeps custom names based on the rootDir', async () => {
+it('keeps custom ids based on the rootDir', async () => {
   const {options} = await normalize(
     {
-      name: 'custom-name',
+      id: 'custom-id',
       rootDir: '/root/path/foo',
     },
     {} as Config.Argv,
   );
 
-  expect(options.name).toBe('custom-name');
+  expect(options.id).toBe('custom-id');
 });
 
 it('minimal config is stable across runs', async () => {
@@ -309,6 +310,102 @@ describe('roots', () => {
   testPathArray('roots');
 });
 
+describe('reporters', () => {
+  let Resolver;
+  beforeEach(() => {
+    Resolver = require('jest-resolve').default;
+    Resolver.findNodeModule = jest.fn(name => name);
+  });
+
+  it('allows empty list', async () => {
+    const {options} = await normalize(
+      {
+        reporters: [],
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(options.reporters).toEqual([]);
+  });
+
+  it('normalizes the path and options object', async () => {
+    const {options} = await normalize(
+      {
+        reporters: [
+          'default',
+          'github-actions',
+          '<rootDir>/custom-reporter.js',
+          ['<rootDir>/custom-reporter.js', {banana: 'yes', pineapple: 'no'}],
+          ['jest-junit', {outputName: 'report.xml'}],
+        ],
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(options.reporters).toEqual([
+      ['default', {}],
+      ['github-actions', {}],
+      ['/root/custom-reporter.js', {}],
+      ['/root/custom-reporter.js', {banana: 'yes', pineapple: 'no'}],
+      ['jest-junit', {outputName: 'report.xml'}],
+    ]);
+  });
+
+  it('throws an error if value is neither string nor array', async () => {
+    expect.assertions(1);
+    await expect(
+      normalize(
+        {
+          reporters: [123],
+          rootDir: '/root/',
+        },
+        {} as Config.Argv,
+      ),
+    ).rejects.toThrowErrorMatchingSnapshot();
+  });
+
+  it('throws an error if first value in the tuple is not a string', async () => {
+    expect.assertions(1);
+    await expect(
+      normalize(
+        {
+          reporters: [[123]],
+          rootDir: '/root/',
+        },
+        {} as Config.Argv,
+      ),
+    ).rejects.toThrowErrorMatchingSnapshot();
+  });
+
+  it('throws an error if second value is missing in the tuple', async () => {
+    expect.assertions(1);
+    await expect(
+      normalize(
+        {
+          reporters: [['some-reporter']],
+          rootDir: '/root/',
+        },
+        {} as Config.Argv,
+      ),
+    ).rejects.toThrowErrorMatchingSnapshot();
+  });
+
+  it('throws an error if second value in the tuple is not an object', async () => {
+    expect.assertions(1);
+    await expect(
+      normalize(
+        {
+          reporters: [['some-reporter', true]],
+          rootDir: '/root/',
+        },
+        {} as Config.Argv,
+      ),
+    ).rejects.toThrowErrorMatchingSnapshot();
+  });
+});
+
 describe('transform', () => {
   let Resolver;
   beforeEach(() => {
@@ -384,7 +481,7 @@ describe('setupFilesAfterEnv', () => {
   beforeEach(() => {
     Resolver = require('jest-resolve').default;
     Resolver.findNodeModule = jest.fn(name =>
-      name.startsWith('/') ? name : '/root/path/foo' + path.sep + name,
+      name.startsWith('/') ? name : `/root/path/foo${path.sep}${name}`,
     );
   });
 
@@ -425,45 +522,6 @@ describe('setupFilesAfterEnv', () => {
   });
 });
 
-describe('setupTestFrameworkScriptFile', () => {
-  let Resolver;
-
-  beforeEach(() => {
-    (console.warn as unknown as jest.SpyInstance).mockImplementation(() => {});
-    Resolver = require('jest-resolve').default;
-    Resolver.findNodeModule = jest.fn(name =>
-      name.startsWith('/') ? name : '/root/path/foo' + path.sep + name,
-    );
-  });
-
-  it('logs a deprecation warning when `setupTestFrameworkScriptFile` is used', async () => {
-    await normalize(
-      {
-        rootDir: '/root/path/foo',
-        setupTestFrameworkScriptFile: 'bar/baz',
-      },
-      {} as Config.Argv,
-    );
-
-    expect(
-      (console.warn as unknown as jest.SpyInstance).mock.calls[0][0],
-    ).toMatchSnapshot();
-  });
-
-  it('logs an error when `setupTestFrameworkScriptFile` and `setupFilesAfterEnv` are used', async () => {
-    await expect(
-      normalize(
-        {
-          rootDir: '/root/path/foo',
-          setupFilesAfterEnv: ['bar/baz'],
-          setupTestFrameworkScriptFile: 'bar/baz',
-        },
-        {} as Config.Argv,
-      ),
-    ).rejects.toThrowErrorMatchingSnapshot();
-  });
-});
-
 describe('coveragePathIgnorePatterns', () => {
   it('does not normalize paths relative to rootDir', async () => {
     // This is a list of patterns, so we can't assume any of them are
@@ -797,7 +855,7 @@ describe('babel-jest', () => {
     Resolver = require('jest-resolve').default;
     Resolver.findNodeModule = jest.fn(name =>
       name.indexOf('babel-jest') === -1
-        ? path.sep + 'node_modules' + path.sep + name
+        ? `${path.sep}node_modules${path.sep}${name}`
         : name,
     );
   });
@@ -831,45 +889,6 @@ describe('babel-jest', () => {
   });
 });
 
-describe('Upgrade help', () => {
-  beforeEach(() => {
-    (console.warn as unknown as jest.SpyInstance).mockImplementation(() => {});
-
-    const Resolver = require('jest-resolve').default;
-    Resolver.findNodeModule = jest.fn(name => {
-      if (name == 'bar/baz') {
-        return '/node_modules/bar/baz';
-      }
-      return findNodeModule(name);
-    });
-  });
-
-  it('logs a warning when `scriptPreprocessor` and/or `preprocessorIgnorePatterns` are used', async () => {
-    const {options: options, hasDeprecationWarnings} = await normalize(
-      {
-        preprocessorIgnorePatterns: ['bar/baz', 'qux/quux'],
-        rootDir: '/root/path/foo',
-        scriptPreprocessor: 'bar/baz',
-      },
-      {} as Config.Argv,
-    );
-
-    expect(options.transform).toEqual([['.*', '/node_modules/bar/baz', {}]]);
-    expect(options.transformIgnorePatterns).toEqual([
-      joinForPattern('bar', 'baz'),
-      joinForPattern('qux', 'quux'),
-    ]);
-
-    expect(options).not.toHaveProperty('scriptPreprocessor');
-    expect(options).not.toHaveProperty('preprocessorIgnorePatterns');
-    expect(hasDeprecationWarnings).toBeTruthy();
-
-    expect(
-      (console.warn as unknown as jest.SpyInstance).mock.calls[0][0],
-    ).toMatchSnapshot();
-  });
-});
-
 describe('testRegex', () => {
   it('testRegex empty string is mapped to empty array', async () => {
     const {options} = await normalize(
@@ -1004,7 +1023,7 @@ describe('preset', () => {
         return null;
       }
 
-      return '/node_modules/' + name;
+      return `/node_modules/${name}`;
     });
     jest.doMock(
       '/node_modules/react-native/jest-preset.json',
@@ -1116,7 +1135,7 @@ describe('preset', () => {
         {} as Config.Argv,
       ),
     ).rejects.toThrowError(
-      /Unexpected token } in JSON at position 104[\s\S]* at /,
+      /Unexpected token } in JSON at position (104|110)[\s\S]* at /,
     );
   });
 
@@ -1304,7 +1323,7 @@ describe('preset with globals', () => {
         return '/node_modules/global-foo/jest-preset.json';
       }
 
-      return '/node_modules/' + name;
+      return `/node_modules/${name}`;
     });
     jest.doMock(
       '/node_modules/global-foo/jest-preset.json',
@@ -1361,7 +1380,7 @@ describe.each(['setupFiles', 'setupFilesAfterEnv'])(
     beforeEach(() => {
       Resolver = require('jest-resolve').default;
       Resolver.findNodeModule = jest.fn(
-        name => path.sep + 'node_modules' + path.sep + name,
+        name => `${path.sep}node_modules${path.sep}${name}`,
       );
     });
 
@@ -1397,6 +1416,55 @@ describe.each(['setupFiles', 'setupFilesAfterEnv'])(
   },
 );
 
+describe("preset with 'reporters' option", () => {
+  beforeEach(() => {
+    const Resolver = require('jest-resolve').default;
+    Resolver.findNodeModule = jest.fn(name => {
+      if (name === 'with-reporters/jest-preset') {
+        return '/node_modules/with-reporters/jest-preset.json';
+      }
+
+      return `/node_modules/${name}`;
+    });
+    jest.doMock(
+      '/node_modules/with-reporters/jest-preset.json',
+      () => ({
+        reporters: ['default'],
+      }),
+      {virtual: true},
+    );
+  });
+
+  afterEach(() => {
+    jest.dontMock('/node_modules/with-reporters/jest-preset.json');
+  });
+
+  test("normalizes 'reporters' option defined in preset", async () => {
+    const {options} = await normalize(
+      {
+        preset: 'with-reporters',
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(options.reporters).toEqual([['default', {}]]);
+  });
+
+  test("overrides 'reporters' option defined in preset", async () => {
+    const {options} = await normalize(
+      {
+        preset: 'with-reporters',
+        reporters: ['summary'],
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(options.reporters).toEqual([['summary', {}]]);
+  });
+});
+
 describe('runner', () => {
   let Resolver;
   beforeEach(() => {
@@ -1559,7 +1627,7 @@ describe('testPathPattern', () => {
   ];
   for (const opt of cliOptions) {
     describe(opt.name, () => {
-      it('uses ' + opt.name + ' if set', async () => {
+      it(`uses ${opt.name} if set`, async () => {
         const argv = {[opt.property]: ['a/b']} as Config.Argv;
         const {options} = await normalize(initialOptions, argv);
 
@@ -1576,13 +1644,20 @@ describe('testPathPattern', () => {
         ).toMatchSnapshot();
       });
 
-      it('joins multiple ' + opt.name + ' if set', async () => {
-        const argv = {testPathPattern: ['a/b', 'c/d']} as Config.Argv;
+      it(`joins multiple ${opt.name} if set`, async () => {
+        const argv = {[opt.property]: ['a/b', 'c/d']} as Config.Argv;
         const {options} = await normalize(initialOptions, argv);
 
         expect(options.testPathPattern).toBe('a/b|c/d');
       });
 
+      it('coerces all patterns to strings', async () => {
+        const argv = {[opt.property]: [1]} as Config.Argv;
+        const {options} = await normalize(initialOptions, argv);
+
+        expect(options.testPathPattern).toBe('1');
+      });
+
       describe('posix', () => {
         it('should not escape the pattern', async () => {
           const argv = {
@@ -1633,13 +1708,6 @@ describe('testPathPattern', () => {
 
           expect(options.testPathPattern).toBe('a\\\\b|c\\\\d');
         });
-
-        it('coerces all patterns to strings', async () => {
-          const argv = {[opt.property]: [1]} as Config.Argv;
-          const {options} = await normalize(initialOptions, argv);
-
-          expect(options.testPathPattern).toBe('1');
-        });
       });
     });
   }
@@ -1668,6 +1736,8 @@ describe('moduleFileExtensions', () => {
 
     expect(options.moduleFileExtensions).toEqual([
       'js',
+      'mjs',
+      'cjs',
       'jsx',
       'ts',
       'tsx',
@@ -1844,19 +1914,22 @@ describe('extensionsToTreatAsEsm', () => {
 describe('haste.enableSymlinks', () => {
   it('should throw if watchman is not disabled', async () => {
     await expect(
-      normalize({haste: {enableSymlinks: true}, rootDir: '/root/'}, {}),
+      normalize(
+        {haste: {enableSymlinks: true}, rootDir: '/root/'},
+        {} as Config.Argv,
+      ),
     ).rejects.toThrow('haste.enableSymlinks is incompatible with watchman');
 
     await expect(
       normalize(
         {haste: {enableSymlinks: true}, rootDir: '/root/', watchman: true},
-        {},
+        {} as Config.Argv,
       ),
     ).rejects.toThrow('haste.enableSymlinks is incompatible with watchman');
 
     const {options} = await normalize(
       {haste: {enableSymlinks: true}, rootDir: '/root/', watchman: false},
-      {},
+      {} as Config.Argv,
     );
 
     expect(options.haste.enableSymlinks).toBe(true);
@@ -1868,10 +1941,174 @@ describe('haste.forceNodeFilesystemAPI', () => {
   it('should pass option through', async () => {
     const {options} = await normalize(
       {haste: {forceNodeFilesystemAPI: true}, rootDir: '/root/'},
-      {},
+      {} as Config.Argv,
     );
 
     expect(options.haste.forceNodeFilesystemAPI).toBe(true);
     expect(console.warn).not.toHaveBeenCalled();
   });
 });
+
+describe('updateSnapshot', () => {
+  it('should be all if updateSnapshot is true', async () => {
+    const {options} = await normalize({rootDir: '/root/'}, {
+      updateSnapshot: true,
+    } as Config.Argv);
+    expect(options.updateSnapshot).toBe('all');
+  });
+  it('should be new if updateSnapshot is falsy', async () => {
+    {
+      const {options} = await normalize(
+        {ci: false, rootDir: '/root/'},
+        {} as Config.Argv,
+      );
+      expect(options.updateSnapshot).toBe('new');
+    }
+    {
+      const {options} = await normalize({ci: false, rootDir: '/root/'}, {
+        updateSnapshot: false,
+      } as Config.Argv);
+      expect(options.updateSnapshot).toBe('new');
+    }
+  });
+  it('should be none if updateSnapshot is falsy and ci mode is true', async () => {
+    const defaultCiConfig = Defaults.ci;
+    {
+      Defaults.ci = false;
+      const {options} = await normalize({rootDir: '/root/'}, {
+        ci: true,
+      } as Config.Argv);
+      expect(options.updateSnapshot).toBe('none');
+    }
+    {
+      Defaults.ci = true;
+      const {options} = await normalize({rootDir: '/root/'}, {} as Config.Argv);
+      expect(options.updateSnapshot).toBe('none');
+    }
+    Defaults.ci = defaultCiConfig;
+  });
+});
+
+describe('shards', () => {
+  it('should be object if defined', async () => {
+    const {options} = await normalize({rootDir: '/root/'}, {
+      shard: '1/2',
+    } as Config.Argv);
+
+    expect(options.shard).toEqual({shardCount: 2, shardIndex: 1});
+  });
+});
+
+describe('logs a deprecation warning', () => {
+  beforeEach(() => {
+    (console.warn as unknown as jest.SpyInstance).mockImplementation(() => {});
+  });
+
+  test("when 'browser' option is passed", async () => {
+    await normalize(
+      {
+        browser: true,
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'extraGlobals' option is passed", async () => {
+    await normalize(
+      {
+        extraGlobals: ['Math'],
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'moduleLoader' option is passed", async () => {
+    await normalize(
+      {
+        moduleLoader: '<rootDir>/runtime.js',
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'preprocessorIgnorePatterns' option is passed", async () => {
+    await normalize(
+      {
+        preprocessorIgnorePatterns: ['/node_modules/'],
+        rootDir: '/root/',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'scriptPreprocessor' option is passed", async () => {
+    await normalize(
+      {
+        rootDir: '/root/',
+        scriptPreprocessor: 'preprocessor.js',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'setupTestFrameworkScriptFile' option is passed", async () => {
+    await normalize(
+      {
+        rootDir: '/root/',
+        setupTestFrameworkScriptFile: 'setup.js',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'testPathDirs' option is passed", async () => {
+    await normalize(
+      {
+        rootDir: '/root/',
+        testPathDirs: ['<rootDir>'],
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'testURL' option is passed", async () => {
+    await normalize(
+      {
+        rootDir: '/root/',
+        testURL: 'https://jestjs.io',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+
+  test("when 'timers' option is passed", async () => {
+    await normalize(
+      {
+        rootDir: '/root/',
+        timers: 'real',
+      },
+      {} as Config.Argv,
+    );
+
+    expect(console.warn).toMatchSnapshot();
+  });
+});
