@@ -13,9 +13,11 @@ function mockHashContents(contents) {
   return crypto.createHash('sha1').update(contents).digest('hex');
 }
 
-jest.mock('child_process', () => ({
-  // If this does not throw, we'll use the (mocked) watchman crawler
-  execSync() {},
+const mockIsWatchmanInstalled = jest.fn().mockResolvedValue(true);
+
+jest.mock('../lib/isWatchmanInstalled', () => ({
+  __esModule: true,
+  default: mockIsWatchmanInstalled,
 }));
 
 jest.mock('jest-worker', () => ({
@@ -31,8 +33,8 @@ jest.mock('jest-worker', () => ({
 }));
 
 jest.mock('../crawlers/node');
-jest.mock('../crawlers/watchman', () =>
-  jest.fn(options => {
+jest.mock('../crawlers/watchman', () => ({
+  watchmanCrawl: jest.fn(options => {
     const path = require('path');
 
     const {data, ignore, rootDir, roots, computeSha1} = options;
@@ -66,7 +68,7 @@ jest.mock('../crawlers/watchman', () =>
       removedFiles,
     });
   }),
-);
+}));
 
 const mockWatcherConstructor = jest.fn(root => {
   const EventEmitter = require('events').EventEmitter;
@@ -206,8 +208,8 @@ describe('HasteMap', () => {
     defaultConfig = {
       extensions: ['js', 'json'],
       hasteImplModulePath: require.resolve('./haste_impl.js'),
+      id: 'haste-map-test',
       maxWorkers: 1,
-      name: 'haste-map-test',
       platforms: ['ios', 'android'],
       resetCache: false,
       rootDir: path.join('/', 'project'),
@@ -241,15 +243,21 @@ describe('HasteMap', () => {
     );
   });
 
-  it('creates different cache file paths for different roots', () => {
+  it('creates different cache file paths for different roots', async () => {
     jest.resetModules();
     const HasteMap = require('../').default;
-    const hasteMap1 = new HasteMap({...defaultConfig, rootDir: '/root1'});
-    const hasteMap2 = new HasteMap({...defaultConfig, rootDir: '/root2'});
+    const hasteMap1 = await HasteMap.create({
+      ...defaultConfig,
+      rootDir: '/root1',
+    });
+    const hasteMap2 = await HasteMap.create({
+      ...defaultConfig,
+      rootDir: '/root2',
+    });
     expect(hasteMap1.getCacheFilePath()).not.toBe(hasteMap2.getCacheFilePath());
   });
 
-  it('creates different cache file paths for different dependency extractor cache keys', () => {
+  it('creates different cache file paths for different dependency extractor cache keys', async () => {
     jest.resetModules();
     const HasteMap = require('../').default;
     const dependencyExtractor = require('./dependencyExtractor');
@@ -258,47 +266,53 @@ describe('HasteMap', () => {
       dependencyExtractor: require.resolve('./dependencyExtractor'),
     };
     dependencyExtractor.setCacheKey('foo');
-    const hasteMap1 = new HasteMap(config);
+    const hasteMap1 = await HasteMap.create(config);
     dependencyExtractor.setCacheKey('bar');
-    const hasteMap2 = new HasteMap(config);
+    const hasteMap2 = await HasteMap.create(config);
     expect(hasteMap1.getCacheFilePath()).not.toBe(hasteMap2.getCacheFilePath());
   });
 
-  it('creates different cache file paths for different values of computeDependencies', () => {
+  it('creates different cache file paths for different values of computeDependencies', async () => {
     jest.resetModules();
     const HasteMap = require('../').default;
-    const hasteMap1 = new HasteMap({
+    const hasteMap1 = await HasteMap.create({
       ...defaultConfig,
       computeDependencies: true,
     });
-    const hasteMap2 = new HasteMap({
+    const hasteMap2 = await HasteMap.create({
       ...defaultConfig,
       computeDependencies: false,
     });
     expect(hasteMap1.getCacheFilePath()).not.toBe(hasteMap2.getCacheFilePath());
   });
 
-  it('creates different cache file paths for different hasteImplModulePath cache keys', () => {
+  it('creates different cache file paths for different hasteImplModulePath cache keys', async () => {
     jest.resetModules();
     const HasteMap = require('../').default;
     const hasteImpl = require('./haste_impl');
     hasteImpl.setCacheKey('foo');
-    const hasteMap1 = new HasteMap(defaultConfig);
+    const hasteMap1 = await HasteMap.create(defaultConfig);
     hasteImpl.setCacheKey('bar');
-    const hasteMap2 = new HasteMap(defaultConfig);
+    const hasteMap2 = await HasteMap.create(defaultConfig);
     expect(hasteMap1.getCacheFilePath()).not.toBe(hasteMap2.getCacheFilePath());
   });
 
-  it('creates different cache file paths for different projects', () => {
+  it('creates different cache file paths for different projects', async () => {
     jest.resetModules();
     const HasteMap = require('../').default;
-    const hasteMap1 = new HasteMap({...defaultConfig, name: '@scoped/package'});
-    const hasteMap2 = new HasteMap({...defaultConfig, name: '-scoped-package'});
+    const hasteMap1 = await HasteMap.create({
+      ...defaultConfig,
+      id: '@scoped/package',
+    });
+    const hasteMap2 = await HasteMap.create({
+      ...defaultConfig,
+      id: '-scoped-package',
+    });
     expect(hasteMap1.getCacheFilePath()).not.toBe(hasteMap2.getCacheFilePath());
   });
 
   it('matches files against a pattern', async () => {
-    const {hasteFS} = await new HasteMap(defaultConfig).build();
+    const {hasteFS} = await (await HasteMap.create(defaultConfig)).build();
     expect(
       hasteFS.matchFiles(
         process.platform === 'win32' ? /project\\fruits/ : /project\/fruits/,
@@ -320,7 +334,7 @@ describe('HasteMap', () => {
     mockFs[path.join('/', 'project', 'fruits', 'Kiwi.js')] = `
       // Kiwi!
     `;
-    const {hasteFS} = await new HasteMap(config).build();
+    const {hasteFS} = await (await HasteMap.create(config)).build();
     expect(hasteFS.matchFiles(/Kiwi/)).toEqual([]);
   });
 
@@ -328,7 +342,7 @@ describe('HasteMap', () => {
     mockFs[path.join('/', 'project', 'fruits', '.git', 'fruit-history.js')] = `
       // test
     `;
-    const {hasteFS} = await new HasteMap(defaultConfig).build();
+    const {hasteFS} = await (await HasteMap.create(defaultConfig)).build();
     expect(hasteFS.matchFiles('.git')).toEqual([]);
   });
 
@@ -341,7 +355,7 @@ describe('HasteMap', () => {
     mockFs[path.join('/', 'project', 'fruits', '.git', 'fruit-history.js')] = `
       // test
     `;
-    const {hasteFS} = await new HasteMap(config).build();
+    const {hasteFS} = await (await HasteMap.create(config)).build();
     expect(hasteFS.matchFiles(/Kiwi/)).toEqual([]);
     expect(hasteFS.matchFiles('.git')).toEqual([]);
   });
@@ -353,7 +367,7 @@ describe('HasteMap', () => {
     `;
 
     try {
-      await new HasteMap(config).build();
+      await (await HasteMap.create(config)).build();
     } catch (err) {
       expect(err.message).toBe(
         'jest-haste-map: the `ignorePattern` option must be a RegExp',
@@ -439,7 +453,7 @@ describe('HasteMap', () => {
       // fbjs2
     `;
 
-    const hasteMap = new HasteMap({
+    const hasteMap = await HasteMap.create({
       ...defaultConfig,
       mocksPattern: '__mocks__',
     });
@@ -517,45 +531,42 @@ describe('HasteMap', () => {
     expect(useBuitinsInContext(hasteMap.read())).toEqual(data);
   });
 
-  it('throws if both symlinks and watchman is enabled', () => {
-    expect(
-      () => new HasteMap({...defaultConfig, enableSymlinks: true}),
-    ).toThrow(
+  it('throws if both symlinks and watchman is enabled', async () => {
+    await expect(
+      HasteMap.create({...defaultConfig, enableSymlinks: true}),
+    ).rejects.toThrow(
       'Set either `enableSymlinks` to false or `useWatchman` to false.',
     );
-    expect(
-      () =>
-        new HasteMap({
-          ...defaultConfig,
-          enableSymlinks: true,
-          useWatchman: true,
-        }),
-    ).toThrow(
+    await expect(
+      HasteMap.create({
+        ...defaultConfig,
+        enableSymlinks: true,
+        useWatchman: true,
+      }),
+    ).rejects.toThrow(
       'Set either `enableSymlinks` to false or `useWatchman` to false.',
     );
 
-    expect(
-      () =>
-        new HasteMap({
-          ...defaultConfig,
-          enableSymlinks: false,
-          useWatchman: true,
-        }),
-    ).not.toThrow();
+    await expect(
+      HasteMap.create({
+        ...defaultConfig,
+        enableSymlinks: false,
+        useWatchman: true,
+      }),
+    ).resolves.not.toThrow();
 
-    expect(
-      () =>
-        new HasteMap({
-          ...defaultConfig,
-          enableSymlinks: true,
-          useWatchman: false,
-        }),
-    ).not.toThrow();
+    await expect(
+      HasteMap.create({
+        ...defaultConfig,
+        enableSymlinks: true,
+        useWatchman: false,
+      }),
+    ).resolves.not.toThrow();
   });
 
   describe('builds a haste map on a fresh cache with SHA-1s', () => {
     it.each([false, true])('uses watchman: %s', async useWatchman => {
-      const node = require('../crawlers/node');
+      const node = require('../crawlers/node').nodeCrawl;
 
       node.mockImplementation(options => {
         const {data} = options;
@@ -603,7 +614,9 @@ describe('HasteMap', () => {
         });
       });
 
-      const hasteMap = new HasteMap({
+      mockIsWatchmanInstalled.mockClear();
+
+      const hasteMap = await HasteMap.create({
         ...defaultConfig,
         computeSha1: true,
         maxWorkers: 1,
@@ -612,6 +625,10 @@ describe('HasteMap', () => {
 
       const data = (await hasteMap.build()).__hasteMapForTest;
 
+      expect(mockIsWatchmanInstalled).toHaveBeenCalledTimes(
+        useWatchman ? 1 : 0,
+      );
+
       expect(data.files).toEqual(
         createMap({
           [path.join('fruits', 'Banana.js')]: [
@@ -666,7 +683,7 @@ describe('HasteMap', () => {
       module.exports = require("./video.mp4");
     `;
 
-    const hasteMap = new HasteMap({
+    const hasteMap = await HasteMap.create({
       ...defaultConfig,
       extensions: [...defaultConfig.extensions],
       roots: [...defaultConfig.roots, path.join('/', 'project', 'video')],
@@ -689,7 +706,7 @@ describe('HasteMap', () => {
       // fbjs!
     `;
 
-    const hasteMap = new HasteMap({
+    const hasteMap = await HasteMap.create({
       ...defaultConfig,
       mocksPattern: '__mocks__',
       retainAllFiles: true,
@@ -738,11 +755,13 @@ describe('HasteMap', () => {
     `;
 
     try {
-      await new HasteMap({
-        mocksPattern: '__mocks__',
-        throwOnModuleCollision: true,
-        ...defaultConfig,
-      }).build();
+      await (
+        await HasteMap.create({
+          mocksPattern: '__mocks__',
+          throwOnModuleCollision: true,
+          ...defaultConfig,
+        })
+      ).build();
     } catch {
       expect(
         console.error.mock.calls[0][0].replace(/\\/g, '/'),
@@ -755,7 +774,9 @@ describe('HasteMap', () => {
       const Banana = require("Banana");
     `;
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
 
     // Duplicate modules are removed so that it doesn't cause
     // non-determinism later on.
@@ -769,10 +790,10 @@ describe('HasteMap', () => {
       const Banana = require("Banana");
     `;
 
-    await new HasteMap(defaultConfig).build();
+    await (await HasteMap.create(defaultConfig)).build();
     expect(console.warn).toHaveBeenCalledTimes(1);
 
-    await new HasteMap(defaultConfig).build();
+    await (await HasteMap.create(defaultConfig)).build();
     expect(console.warn).toHaveBeenCalledTimes(1);
   });
 
@@ -784,10 +805,12 @@ describe('HasteMap', () => {
     `;
 
     try {
-      await new HasteMap({
-        throwOnModuleCollision: true,
-        ...defaultConfig,
-      }).build();
+      await (
+        await HasteMap.create({
+          throwOnModuleCollision: true,
+          ...defaultConfig,
+        })
+      ).build();
     } catch (err) {
       expect(err.message).toBe(
         'Duplicated files or mocks. Please check the console for more info',
@@ -809,7 +832,9 @@ describe('HasteMap', () => {
       const Blackberry = require("Blackberry");
     `;
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
 
     expect(data.files).toEqual(
       createMap({
@@ -855,8 +880,8 @@ describe('HasteMap', () => {
   });
 
   it('does not access the file system on a warm cache with no changes', async () => {
-    const {__hasteMapForTest: initialData} = await new HasteMap(
-      defaultConfig,
+    const {__hasteMapForTest: initialData} = await (
+      await HasteMap.create(defaultConfig)
     ).build();
 
     // The first run should access the file system once for the (empty)
@@ -874,7 +899,9 @@ describe('HasteMap', () => {
       vegetables: 'c:fake-clock:4',
     });
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
     expect(fs.readFileSync.mock.calls.length).toBe(1);
     if (require('v8').deserialize) {
       expect(fs.readFileSync).toBeCalledWith(cacheFilePath);
@@ -887,8 +914,8 @@ describe('HasteMap', () => {
   });
 
   it('only does minimal file system access when files change', async () => {
-    const {__hasteMapForTest: initialData} = await new HasteMap(
-      defaultConfig,
+    const {__hasteMapForTest: initialData} = await (
+      await HasteMap.create(defaultConfig)
     ).build();
     fs.readFileSync.mockClear();
 
@@ -905,7 +932,9 @@ describe('HasteMap', () => {
       vegetables: 'c:fake-clock:2',
     });
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
 
     expect(fs.readFileSync.mock.calls.length).toBe(2);
 
@@ -938,8 +967,8 @@ describe('HasteMap', () => {
   });
 
   it('correctly handles file deletions', async () => {
-    const {__hasteMapForTest: initialData} = await new HasteMap(
-      defaultConfig,
+    const {__hasteMapForTest: initialData} = await (
+      await HasteMap.create(defaultConfig)
     ).build();
     fs.readFileSync.mockClear();
 
@@ -955,7 +984,9 @@ describe('HasteMap', () => {
       vegetables: 'c:fake-clock:2',
     });
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
 
     const files = new Map(initialData.files);
     files.delete(path.join('fruits', 'Banana.js'));
@@ -972,7 +1003,9 @@ describe('HasteMap', () => {
       const Banana = require("Banana");
     `;
     let data;
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       g: [path.join('fruits', 'Strawberry.js'), 0],
     });
@@ -984,7 +1017,9 @@ describe('HasteMap', () => {
       `,
     });
     mockClocks = createMap({fruits: 'c:fake-clock:3'});
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       g: [path.join('fruits', 'Strawberry.js'), 0],
       ios: [path.join('fruits', 'Strawberry.ios.js'), 0],
@@ -1000,7 +1035,9 @@ describe('HasteMap', () => {
       const Raspberry = require("Raspberry");
     `;
     let data;
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       g: [path.join('fruits', 'Strawberry.js'), 0],
       ios: [path.join('fruits', 'Strawberry.ios.js'), 0],
@@ -1011,7 +1048,9 @@ describe('HasteMap', () => {
       [path.join('/', 'project', 'fruits', 'Strawberry.ios.js')]: null,
     });
     mockClocks = createMap({fruits: 'c:fake-clock:3'});
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       g: [path.join('fruits', 'Strawberry.js'), 0],
     });
@@ -1023,7 +1062,9 @@ describe('HasteMap', () => {
       const Raspberry = require("Raspberry");
     `;
     let data;
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       ios: [path.join('fruits', 'Strawberry.ios.js'), 0],
     });
@@ -1036,7 +1077,9 @@ describe('HasteMap', () => {
       `,
     });
     mockClocks = createMap({fruits: 'c:fake-clock:3'});
-    ({__hasteMapForTest: data} = await new HasteMap(defaultConfig).build());
+    ({__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build());
     expect(data.map.get('Strawberry')).toEqual({
       g: [path.join('fruits', 'Strawberry.js'), 0],
     });
@@ -1050,8 +1093,8 @@ describe('HasteMap', () => {
         const Blackberry = require("Blackberry");
       `;
 
-      const {__hasteMapForTest: data} = await new HasteMap(
-        defaultConfig,
+      const {__hasteMapForTest: data} = await (
+        await HasteMap.create(defaultConfig)
       ).build();
       expect(useBuitinsInContext(data.duplicates)).toEqual(
         createMap({
@@ -1078,8 +1121,8 @@ describe('HasteMap', () => {
         vegetables: 'c:fake-clock:2',
       });
 
-      const {__hasteMapForTest: data} = await new HasteMap(
-        defaultConfig,
+      const {__hasteMapForTest: data} = await (
+        await HasteMap.create(defaultConfig)
       ).build();
       expect(useBuitinsInContext(data.duplicates)).toEqual(new Map());
       expect(data.map.get('Strawberry')).toEqual({
@@ -1098,8 +1141,8 @@ describe('HasteMap', () => {
         {"name": "Strawberry"}
       `;
 
-      const {__hasteMapForTest: data} = await new HasteMap(
-        defaultConfig,
+      const {__hasteMapForTest: data} = await (
+        await HasteMap.create(defaultConfig)
       ).build();
 
       expect(useBuitinsInContext(data.duplicates)).toEqual(
@@ -1136,8 +1179,8 @@ describe('HasteMap', () => {
         fruits: 'c:fake-clock:4',
       });
 
-      const {__hasteMapForTest: correctData} = await new HasteMap(
-        defaultConfig,
+      const {__hasteMapForTest: correctData} = await (
+        await HasteMap.create(defaultConfig)
       ).build();
 
       expect(useBuitinsInContext(correctData.duplicates)).toEqual(new Map());
@@ -1158,8 +1201,8 @@ describe('HasteMap', () => {
         vegetables: 'c:fake-clock:2',
       });
 
-      const {__hasteMapForTest: data} = await new HasteMap(
-        defaultConfig,
+      const {__hasteMapForTest: data} = await (
+        await HasteMap.create(defaultConfig)
       ).build();
       expect(useBuitinsInContext(data.duplicates)).toEqual(new Map());
       expect(data.map.get('Strawberry')).toEqual({
@@ -1177,7 +1220,7 @@ describe('HasteMap', () => {
 
   it('discards the cache when configuration changes', async () => {
     HasteMap.getCacheFilePath = getCacheFilePath;
-    await new HasteMap(defaultConfig).build();
+    await (await HasteMap.create(defaultConfig)).build();
     fs.readFileSync.mockClear();
 
     // Explicitly mock that no files have changed.
@@ -1190,12 +1233,12 @@ describe('HasteMap', () => {
     });
 
     const config = {...defaultConfig, ignorePattern: /Kiwi|Pear/};
-    const {moduleMap} = await new HasteMap(config).build();
+    const {moduleMap} = await (await HasteMap.create(config)).build();
     expect(moduleMap.getModule('Pear')).toBe(null);
   });
 
   it('ignores files that do not exist', async () => {
-    const watchman = require('../crawlers/watchman');
+    const watchman = require('../crawlers/watchman').watchmanCrawl;
     const mockImpl = watchman.getMockImplementation();
     // Wrap the watchman mock and add an invalid file to the file list.
     watchman.mockImplementation(options =>
@@ -1212,7 +1255,9 @@ describe('HasteMap', () => {
       }),
     );
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
     expect(data.files.size).toBe(5);
 
     // Ensure this file is not part of the file list.
@@ -1225,12 +1270,14 @@ describe('HasteMap', () => {
     const jestWorker = require('jest-worker').Worker;
     const path = require('path');
     const dependencyExtractor = path.join(__dirname, 'dependencyExtractor.js');
-    const {__hasteMapForTest: data} = await new HasteMap({
-      ...defaultConfig,
-      dependencyExtractor,
-      hasteImplModulePath: undefined,
-      maxWorkers: 4,
-    }).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create({
+        ...defaultConfig,
+        dependencyExtractor,
+        hasteImplModulePath: undefined,
+        maxWorkers: 4,
+      })
+    ).build();
 
     expect(jestWorker.mock.calls.length).toBe(1);
 
@@ -1293,8 +1340,8 @@ describe('HasteMap', () => {
   });
 
   it('tries to crawl using node as a fallback', async () => {
-    const watchman = require('../crawlers/watchman');
-    const node = require('../crawlers/node');
+    const watchman = require('../crawlers/watchman').watchmanCrawl;
+    const node = require('../crawlers/node').nodeCrawl;
 
     watchman.mockImplementation(() => {
       throw new Error('watchman error');
@@ -1310,7 +1357,9 @@ describe('HasteMap', () => {
       });
     });
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
     expect(watchman).toBeCalled();
     expect(node).toBeCalled();
 
@@ -1331,8 +1380,8 @@ describe('HasteMap', () => {
   });
 
   it('tries to crawl using node as a fallback when promise fails once', async () => {
-    const watchman = require('../crawlers/watchman');
-    const node = require('../crawlers/node');
+    const watchman = require('../crawlers/watchman').watchmanCrawl;
+    const node = require('../crawlers/node').nodeCrawl;
 
     watchman.mockImplementation(() =>
       Promise.reject(new Error('watchman error')),
@@ -1348,7 +1397,9 @@ describe('HasteMap', () => {
       });
     });
 
-    const {__hasteMapForTest: data} = await new HasteMap(defaultConfig).build();
+    const {__hasteMapForTest: data} = await (
+      await HasteMap.create(defaultConfig)
+    ).build();
 
     expect(watchman).toBeCalled();
     expect(node).toBeCalled();
@@ -1369,8 +1420,8 @@ describe('HasteMap', () => {
 
   it('stops crawling when both crawlers fail', async () => {
     expect.assertions(1);
-    const watchman = require('../crawlers/watchman');
-    const node = require('../crawlers/node');
+    const watchman = require('../crawlers/watchman').watchmanCrawl;
+    const node = require('../crawlers/node').nodeCrawl;
 
     watchman.mockImplementation(() =>
       Promise.reject(new Error('watchman error')),
@@ -1381,7 +1432,7 @@ describe('HasteMap', () => {
     );
 
     try {
-      await new HasteMap(defaultConfig).build();
+      await (await HasteMap.create(defaultConfig)).build();
     } catch (error) {
       expect(error.message).toEqual(
         'Crawler retry failed:\n' +
@@ -1410,7 +1461,7 @@ describe('HasteMap', () => {
           mockFs = options.mockFs;
         }
         const watchConfig = {...defaultConfig, watch: true};
-        const hm = new HasteMap(watchConfig);
+        const hm = await HasteMap.create(watchConfig);
         await hm.build();
         try {
           await fn(hm);
