@@ -30,6 +30,17 @@ describe('moduleMocker', () => {
       expect(metadata.name).toBe('x');
     });
 
+    it('does not return broken name property', () => {
+      class By {
+        static name() {
+          return 'this is not a name';
+        }
+      }
+      const metadata = moduleMocker.getMetadata(By);
+      expect(typeof By.name).toBe('function');
+      expect(metadata).not.toHaveProperty('name');
+    });
+
     it('mocks constant values', () => {
       const metadata = moduleMocker.getMetadata(Symbol.for('bowties.are.cool'));
       expect(metadata.value).toEqual(Symbol.for('bowties.are.cool'));
@@ -424,20 +435,53 @@ describe('moduleMocker', () => {
         expect(fn.mock.instances[1]).toBe(instance2);
       });
 
+      it('tracks context objects passed to mock calls', () => {
+        const fn = moduleMocker.fn();
+        expect(fn.mock.instances).toEqual([]);
+
+        const ctx0 = {};
+        fn.apply(ctx0, []);
+        expect(fn.mock.contexts[0]).toBe(ctx0);
+
+        const ctx1 = {};
+        fn.call(ctx1);
+        expect(fn.mock.contexts[1]).toBe(ctx1);
+
+        const ctx2 = {};
+        const bound2 = fn.bind(ctx2);
+        bound2();
+        expect(fn.mock.contexts[2]).toBe(ctx2);
+
+        // null context
+        fn.apply(null, []); // eslint-disable-line no-useless-call
+        expect(fn.mock.contexts[3]).toBe(null);
+        fn.call(null); // eslint-disable-line no-useless-call
+        expect(fn.mock.contexts[4]).toBe(null);
+        fn.bind(null)();
+        expect(fn.mock.contexts[5]).toBe(null);
+
+        // Unspecified context is `undefined` in strict mode (like in this test) and `window` otherwise.
+        fn();
+        expect(fn.mock.contexts[6]).toBe(undefined);
+      });
+
       it('supports clearing mock calls', () => {
         const fn = moduleMocker.fn();
         expect(fn.mock.calls).toEqual([]);
 
         fn(1, 2, 3);
         expect(fn.mock.calls).toEqual([[1, 2, 3]]);
+        expect(fn.mock.contexts).toEqual([undefined]);
 
         fn.mockReturnValue('abcd');
 
         fn.mockClear();
         expect(fn.mock.calls).toEqual([]);
+        expect(fn.mock.contexts).toEqual([]);
 
         fn('a', 'b', 'c');
         expect(fn.mock.calls).toEqual([['a', 'b', 'c']]);
+        expect(fn.mock.contexts).toEqual([undefined]);
 
         expect(fn()).toEqual('abcd');
       });
@@ -553,7 +597,7 @@ describe('moduleMocker', () => {
 
       moduleMocker.spyOn(child, 'func').mockReturnValue('efgh');
 
-      expect(child.hasOwnProperty('func')).toBe(true);
+      expect(Object.prototype.hasOwnProperty.call(child, 'func')).toBe(true);
       expect(child.func()).toEqual('efgh');
       expect(parent.func()).toEqual('abcd');
     });
@@ -569,7 +613,7 @@ describe('moduleMocker', () => {
 
       moduleMocker.spyOn(parent, 'func').mockReturnValue('jklm');
 
-      expect(child.hasOwnProperty('func')).toBe(false);
+      expect(Object.prototype.hasOwnProperty.call(child, 'func')).toBe(false);
       expect(child.func()).toEqual('jklm');
     });
 
@@ -1124,6 +1168,7 @@ describe('moduleMocker', () => {
       const obj = {
         method() {
           isOriginalCalled = true;
+          // eslint-disable-next-line @typescript-eslint/no-this-alias
           originalCallThis = this;
           originalCallArguments = arguments;
         },
@@ -1211,6 +1256,7 @@ describe('moduleMocker', () => {
         get method() {
           return function () {
             isOriginalCalled = true;
+            // eslint-disable-next-line @typescript-eslint/no-this-alias
             originalCallThis = this;
             originalCallArguments = arguments;
           };
@@ -1265,6 +1311,7 @@ describe('moduleMocker', () => {
         get method() {
           return function () {
             isOriginalCalled = true;
+            // eslint-disable-next-line @typescript-eslint/no-this-alias
             originalCallThis = this;
             originalCallArguments = arguments;
           };
@@ -1379,6 +1426,7 @@ describe('moduleMocker', () => {
         get method() {
           return function () {
             isOriginalCalled = true;
+            // eslint-disable-next-line @typescript-eslint/no-this-alias
             originalCallThis = this;
             originalCallArguments = arguments;
           };
