@@ -10,5 +10,5 @@
 module.exports.process = source => {
   const json = JSON.parse(source);
   Object.keys(json).forEach(k => (json[k] = k));
-  return JSON.stringify(json);
+  return {code: JSON.stringify(json)};
 };
