@@ -6,9 +6,8 @@
  */
 
 import {realpathSync} from 'graceful-fs';
-import type {Config} from '@jest/types';
 
-export default function tryRealpath(path: Config.Path): Config.Path {
+export default function tryRealpath(path: string): string {
   try {
     path = realpathSync.native(path);
   } catch (error: any) {
